function [p_outl, stud_res, p_cor, cor] = rsresid(y, X, additional, savepdf)
% RSRESID - Check residual for model adequacy. Version 0.1.5.
%
% Outliers are detected using the externally studentized residual if the number of experiments are at least two more than the number of parameters in the model.
% The residual is plotted against the normal distribution. Also calculate and plot the residual against time, the response predicted by the model, the measured response, and any requested additional vectors.
%
% The syntax is
% [p_outl, stud_res, p_cor, cor] = rsresid(y, X, [additional])
%
% Arguments
% y is an N by 1 column vector of data to fit.
% X is an N by a matrix containing the values of a predictors used in the regression, including a possible constant 1 column.
% additional is a matrix whose columns are any variables against which one wants to calculate correlation and plot the residual. This matrix may be omitted.
%
% Return values (all are column vectors)
% p_outl are the p-values for the nullhypotheses that the observation with the corresponding index in y is not an outlier, based on the R-studentized residual. This has the same length N as the response vector y. If N-1-a>0 is false, then this p_outl is zero.
% stud_res is the studentized residual itself if N-1-a>0, otherwise zero.
% p_cor are the p-values for the null hypotheses that the residual is uncorrelated with [time (1:N)', prediction X*b, response y, any variables in additional]. This vector has length 2+n, where n is the width of the optional matrix additional. 
% cor are the correlation coefficients corresponding to the p values in p_cor.
%
% References
% For background, see Section 2.7 in Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009.


% Copyright (C) 2015 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2015


% Check input
if nargin < 2 || nargin > 4,
    error('Wrong number of arguments. Try help rsresid for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsresid for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsresid for instructions.');
end
if cond(X'*X) > 1e10,
    error('The normal equation is ill-conditioned. If the matrix X does not have linearly dependent columns, try standardizing the factor data.');
end

% Standard regression
[N,a]=size(X);
b=X\y;
pred=X*b;
r=y-pred;
SSE=r'*r;

% Check for outliers using externally studentized 
% residual, if possible. Otherwise output zero.
p_outl=0;
stud_res=0;

if N-1-a > 0,
  H=X*(X'*X)^-1*X';
  hii=diag(H);
  Ssq=( SSE -r.^2./(1-hii) ) / (N-1-a);
  stud_res=r./sqrt(Ssq.*(1-hii));
  p_outl=2*tcdf(-abs(stud_res),N-1-a);
end
  
% Check that the residual is normally distributed
figure('PaperPosition',[0 0 12 8],'PaperSize',[12 8]);
normplot(r)
if nargin == 4,
   print('rsresid-normplot','-dpdf')
end
figure('PaperPosition',[0 0 12 8],'PaperSize',[12 8]);
histfit(r)
title('Normal distribution fitted to histogram')
xlabel('Interval')
ylabel('Frequency')
if nargin == 4,
   print('rsresid-histfit','-dpdf')
end

% Calculate and plot correlations
Time=(1:N)';
[cor(1,1), p_cor(1,1)]=corr(r,Time);
[cor(2,1), p_cor(2,1)]=corr(r,pred);
[cor(3,1), p_cor(3,1)]=corr(r,y);

figure('PaperPosition',[0 0 12 8],'PaperSize',[12 8]);
plot(Time,r,'x')
title('Residual vs. time')
xlabel('Time 1:N (table order of X)')
ylabel('Residual')
xlim([min(Time)-1, max(Time)+1]);
if nargin == 4,
   print('rsresid-time','-dpdf')
end
figure('PaperPosition',[0 0 12 8],'PaperSize',[12 8]);
plot(pred,r,'x')
title('Residual vs. prediction')
xlabel('Prediction X*b of regression model')
ylabel('Residual')
xlim([min(pred)-1, max(pred)+1]);
if nargin == 4,
   print('rsresid-pred','-dpdf')
end
figure('PaperPosition',[0 0 12 8],'PaperSize',[12 8]);
plot(y,r,'x')
title('Residual vs. measured response')
xlabel('Measured response y')
ylabel('Residual')
xlim([min(y)-1, max(y)+1]);
if nargin == 4,
   print('rsresid-resp','-dpdf')
end

% Do the same for additional predictors, if given
if nargin>2,
  [~, n]=size(additional); 
  for i=1:n,
    vari=additional(:,i);
    [cor(2+i,1), p_cor(2+i,1)]=corr(r,vari);
    figure('PaperPosition',[0 0 12 8],'PaperSize',[12 8]);
    plot(vari,r,'x')
    title(['Residual vs. additional predictor #',int2str(i)])
    xlabel(['Additional predictor #',int2str(i)])
    ylabel('Residual')
    xlim([min(vari)-1, max(vari)+1]);
    if nargin == 4,
      print(['rsresid-add',int2str(i)],'-dpdf')
    end
  end
end

end